#!/bin/sh

#
# Print script usage and exit
#
usage(){
  local flag=$1
  local self=`basename $0`
  cat <<HERE
Rebuild files.mk, which contains the "pages" make target and the
individual file make targets.

Usage: ${self} <options>

Options:

-h or -?      - Show this help text and exit without error.
-s <src_dir>  - Directory containing source files to convert.
-d <dest_dir> - Directory containing the converted destination files.
-t <temp_dir> - Directory to use for temporary storage.

Directory options are either absolute paths or paths relative to the
current directory, not the directory in which ${self} lives.

HERE
  if [ -n "${flag}" ]; then exit 0; else exit 1; fi
}

#
# Basic tools check
#
which -s which || { echo "No 'which' command."; exit 1; }
which -s basename || { echo "No 'basename' command."; exit 1; }
which -s dirname || { echo "No 'dirname' command."; exit 1; }
which -s mktemp || { echo "No 'mktemp' command."; exit 1; }
[ 0 -eq 0 ]    || { echo "The '[' command doesn't work."; exit 1; } # ]
which -s grep  || { echo "No 'grep' command."; exit 1; }
which -s sed   || { echo "No 'sed' command."; exit 1; }
which -s cat   || { echo "No 'cat' command."; exit 1; }
which -s tr    || { echo "No 'tr' command."; exit 1; }
which -s find  || { echo "No 'find' command."; exit 1; }

#
# Process command line options
#
unset SRC_DIR DST_DIR TMP_DIR FILE

while getopts 's:d:t:h?' c
  do
    case $c in
      s) SRC_DIR=${OPTARG} ;;
      d) DST_DIR=${OPTARG} ;;
      t) TMP_DIR=${OPTARG} ;;
      h)                   ;&
      ?) usage help        ;;
    esac
  done
if [ -z "${SRC_DIR}" ]; then echo "No source dir provided."; usage; fi
if [ -z "${DST_DIR}" ]; then echo "No destination dir provided."; usage; fi
if [ -z "${TMP_DIR}" ]; then echo "No temp dir provided."; usage; fi

#
# Loop prep
#
c=0
TMP_WORK=`mktemp -d ${TMP_DIR}/makefiles.XXXXXX`
[ $? -eq 0 ] || { echo "mktemp err: '${TMP_WORK}'"; exit 1; }
cat /dev/null > ${TMP_WORK}/alltarget
cat /dev/null > ${TMP_WORK}/septargets
[ -f ${TMP_WORK}/alltarget -a -f ${TMP_WORK}/septargets ] || \
  { echo "Couldn't make temp files in ${TMP_WORK}"; exit 1; }

build="\tbin/txt2html.sh"
build="${build} -s \"\${SRC_DIR}\""
build="${build} -d \"\${DST_DIR}\""
build="${build} -t \"\${TMP_DIR}\""
build="${build} -i \"\${TPL_DIR}\""
build="${build} -x \"\${XLAT}\""
build="${build} -1 \"\${XLAT_OPTS1}\""
build="${build} -2 \"\${XLAT_OPTS2}\""
build="${build} -3 \"\${XLAT_OPTS3}\""
build="${build} -f"

echo "    File targets found:"

#
# Iterate over every source file found
#
for file in `find ${SRC_DIR} -type f -name "*.txt"`
  do
    c=$((c+1))

    #
    # Discover file names, strip source-dir name.
    #
    fname=`echo ${file} | sed -n "s:${SRC_DIR}/::;p"`
    fbase=`echo ${fname} | sed -n "s/.txt$//;p"`
    tname="${fbase}.html"

    #
    # Discover template name.
    #
    sed -n "1,/^$/p" ${file} > ${TMP_WORK}/header
    grep -e "^%tpl " ${TMP_WORK}/header | \
    sed -n "s/^%tpl //;p" > ${TMP_WORK}/template
    TEMPLATE=`cat ${TMP_WORK}/template | tr -d "\n"`
    if [ -z "${TEMPLATE}" ]
      then
        TEMPLATE="page.m4"
        echo -e "${c}\t${DST_DIR}/${tname}"
      else
        echo -e "${c}\t${DST_DIR}/${tname}\t\t${TEMPLATE}"
      fi

    #
    # Add to targets lists.
    #
    tfname="\${DST_DIR}/${tname}"

    #
    # Special case: The sitemap page depends on sitemap.m4 in the same
    # directory as the Makefile.  Make sure it's added as a prereq.
    #
    if [ "${tname}" = "sitemap/index.html" ]
      then
        sfname="\${SRC_DIR}/${fname} \${TPL_DIR}/${TEMPLATE} sitemap.m4"
      else
        sfname="\${SRC_DIR}/${fname} \${TPL_DIR}/${TEMPLATE}"
      fi

    echo -e "pages: ${tfname}" >> ${TMP_WORK}/alltarget
    echo -e "${tfname}: ${sfname}" >> ${TMP_WORK}/septargets
    echo -e "${build} \"${fname}\"" >> ${TMP_WORK}/septargets
    echo "" >> ${TMP_WORK}/septargets
  done

#
# Combine into included files.mk list
#
rm -f files.mk
echo "#" > files.mk
echo "# Make targets for each source file." >> files.mk
echo "#" >> files.mk
echo "# DO NOT EDIT!" >> files.mk
echo "# This file is overwritten by the \"files\" make target." >> files.mk
echo "#" >> files.mk
echo "" >> files.mk
cat ${TMP_WORK}/alltarget >> files.mk
echo "" >> files.mk
echo "" >> files.mk
cat ${TMP_WORK}/septargets >> files.mk
echo "" >> files.mk
[ files.mk -nt ${TMP_WORK}/alltarget ] || \
  { echo "Couldn't write to files.mk"; exit 1; }

#
# Clean up
#
rm -f ${TMP_WORK}/*
rmdir ${TMP_WORK}
[ -d ${TMP_WORK} ] && \
  { echo "WARNING: ${TMP_WORK} couldn't be removed."; }
echo ""
echo "    File targets made.  Make the \"pages\" target to build them."
echo "done."

