#!/bin/sh

#
# Print script usage and exit
#
usage(){
  local flag=$1
  local self=`basename $0`
  cat << HERE
Create and populate directories for a new static website.

Usage: ${self} <options>

Options:

-h or -?      - Show this help text and exit without error.
-s <src_dir>  - Directory to contain source files for static website.
-d <dest_dir> - Staging directory for the generated and synced website.
-l <tpl_dir>  - Directory to contain template files for conversion.
-p <pub_dir>  - Directory with public no-compile files.
-t <temp_dir> - Directory to use for temporary storage.

Directory options are either absolute paths or paths relative to the
current directory, not the directory in which ${self} lives.

HERE
  if [ -n "${flag}" ]; then exit 0; else exit 1; fi
}

#
# Basic tools check
#
which -s which || { echo "No 'which' command."; exit 1; }
which -s tar   || { echo "No 'tar' command."; exit 1; }
which -s basename || { echo "No 'basename' command."; exit 1; }
which -s mkdir || { echo "No 'mkdir' command."; exit 1; }
[ 0 -eq 0 ]    || { echo "The '[' command doesn't work."; exit 1; } # ]

#
# Process command line options
#
unset SRC_DIR DST_DIR TPL_DIR PUB_DIR TMP_DIR

while getopts 's:d:l:p:t:h?' c
  do
    case $c in
      s) SRC_DIR=${OPTARG} ;;
      d) DST_DIR=${OPTARG} ;;
      l) TPL_DIR=${OPTARG} ;;
      p) PUB_DIR=${OPTARG} ;;
      t) TMP_DIR=${OPTARG} ;;
      h)                   ;&
      ?) usage help        ;;
    esac
  done
if [ -z "${SRC_DIR}" ]; then echo "No source dir provided."; usage; fi
if [ -z "${DST_DIR}" ]; then echo "No destination dir provided."; usage; fi
if [ -z "${TPL_DIR}" ]; then echo "No template dir provided."; usage; fi
if [ -z "${PUB_DIR}" ]; then echo "No public dir provided."; usage; fi
if [ -z "${TMP_DIR}" ]; then echo "No temporary dir provided."; usage; fi

#
# Check that each directory exits.  If not, make it.
# If it exists but is not a directory, show an error message and quit.
#
if [ -e "${SRC_DIR}" ]
  then
    # it exists
    if [ -d "${SRC_DIR}" ]
      then
        # it's a directory
        echo "SRC_DIR directory ${SRC_DIR} already exists."
      else
        # it's a type of file other than a directory
        echo "SRC_DIR ${SRC_DIR} already exists but is not a directory."
        echo "Edit config.mk and set SRC_DIR to a directory name."
        exit 1
      fi # [ -d "${SRC_DIR}" ]
    #
  else
    # it doesn't exist
    echo -n "Creating SRC_DIR ${SRC_DIR} ... "
    mkdir -p "${SRC_DIR}"
    if [ $? -eq 0 ]
      then
        #
        # Create template /index.html source file
        #
        (
          cd ${SRC_DIR}
          tar xzf ../files_src_dir.tar.gz
          cd ..
        )
        echo "done."
      else
        echo "Error $?"
        exit 1
      fi # mkdir return code
    #
  fi # [ -e "${SRC_DIR} ]

if [ -e "${DST_DIR}" ]
  then
    # it exists
    if [ -d "${DST_DIR}" ]
      then
        # it's a directory
        echo "DST_DIR directory ${DST_DIR} already exists."
      else
        # it's a type of file other than a directory
        echo "DST_DIR ${DST_DIR} already exists but is not a directory."
        echo "Edit config.mk and set DST_DIR to a directory name."
        exit 1
      fi # [ -d "${DST_DIR}" ]
    #
  else
    # it doesn't exist
    echo -n "Creating DST_DIR ${DST_DIR} ... "
    mkdir -p "${DST_DIR}"
    if [ $? -eq 0 ]
      then
        echo "done."
      else
        echo "Error $?"
        exit 1
      fi # mkdir return code
    #
  fi # [ -e "${DST_DIR} ]
  
if [ -e "${TPL_DIR}" ]
  then
    # it exists
    if [ -d "${TPL_DIR}" ]
      then
        # it's a directory
        echo "TPL_DIR directory ${TPL_DIR} already exists."
      else
        # it's a type of file other than a directory
        echo "TPL_DIR ${TPL_DIR} already exists but is not a directory."
        echo "Edit config.mk and set TPL_DIR to a directory name."
        exit 1
      fi # [ -d "${TPL_DIR}" ]
    #
  else
    # it doesn't exist
    echo -n "Creating TPL_DIR ${TPL_DIR} ... "
    mkdir -p "${TPL_DIR}"
    if [ $? -eq 0 ]
      then
        (
          cd ${TPL_DIR}
          tar xzf ../files_tpl_dir.tar.gz
          cd ..
        )
        echo "done."
      else
        echo "Error $?"
        exit 1
      fi # mkdir return code
    #
  fi # [ -e "${TPL_DIR} ]
  
if [ -e "${PUB_DIR}" ]
  then
    # it exists
    if [ -d "${PUB_DIR}" ]
      then
        # it's a directory
        echo "PUB_DIR directory ${PUB_DIR} already exists."
      else
        # it's a type of file other than a directory
        echo "PUB_DIR ${PUB_DIR} already exists but is not a directory."
        echo "Edit config.mk and set PUB_DIR to a directory name."
        exit 1
      fi # [ -d "${PUB_DIR}" ]
    #
  else
    # it doesn't exist
    echo -n "Creating PUB_DIR ${PUB_DIR} ... "
    mkdir -p "${PUB_DIR}"
    if [ $? -eq 0 ]
      then
        (
          cd ${PUB_DIR}
          tar xzf ../files_pub_dir.tar.gz
          cd ..
        )
        echo "done."
      else
        echo "Error $?"
        exit 1
      fi # mkdir return code
    #
  fi # [ -e "${PUB_DIR} ]
  
if [ -e "${TMP_DIR}" ]
  then
    # it exists
    if [ -d "${TMP_DIR}" ]
      then
        # it's a directory
        echo "TMP_DIR directory ${TMP_DIR} already exists."
      else
        # it's a type of file other than a directory
        echo "TMP_DIR ${TMP_DIR} already exists but is not a directory."
        echo "Edit config.mk and set TMP_DIR to a directory name."
        exit 1
      fi # [ -d "${TMP_DIR}" ]
    #
  else
    # it doesn't exist
    echo -n "Creating TMP_DIR ${TMP_DIR} ... "
    mkdir -p "${TMP_DIR}"
    if [ $? -eq 0 ]
      then
        echo "done."
      else
        echo "Error $?"
        exit 1
      fi # mkdir return code
    #
  fi # [ -e "${TMP_DIR} ]
  
