#!/bin/sh

#
# Print script usage and exit
#
usage(){
  local flag=$1
  local self=`basename $0`
  cat << HERE
Create sitemap source page in <src_dir>/sitemap/index.txt.

Usage: ${self} <options>

Options:

-h or -?      - Show this help text and exit without error.
-s <src_dir>  - Directory containing source files to map.
-t <temp_dir> - Directory to use for temporary storage.
-m <sitemap>  - Title for the sitemap page.

Directory options are either absolute paths or paths relative to the
current directory, not the directory in which ${self} lives.

HERE
  if [ -n "${flag}" ]; then exit 0; else exit 1; fi
}

#
# Basic tools check
#
which -s which || { echo "No 'which' command."; exit 1; }
which -s basename || { echo "No 'basename' command."; exit 1; }
which -s mktemp || { echo "No 'mktemp' command."; exit 1; }
[ 0 -eq 0 ]    || { echo "The '[' command doesn't work."; exit 1; } # ]
which -s grep  || { echo "No 'grep' command."; exit 1; }
which -s sed   || { echo "No 'sed' command."; exit 1; }
which -s cat   || { echo "No 'cat' command."; exit 1; }
which -s tr    || { echo "No 'tr' command."; exit 1; }
which -s find  || { echo "No 'find' command."; exit 1; }
which -s sort  || { echo "No 'sort' command."; exit 1; }

#
# Process command line options
#
unset SRC_DIR TMP_DIR TITLE

while getopts 's:t:m:h?' c
  do
    case $c in
      s) SRC_DIR=${OPTARG} ;;
      t) TMP_DIR=${OPTARG} ;;
      m) TITLE=${OPTARG}   ;;
      h)                   ;&
      ?) usage help        ;;
    esac
  done
if [ -z "${SRC_DIR}" ]; then echo "No source dir provided."; usage; fi
if [ -z "${TMP_DIR}" ]; then echo "No temp dir provided."; usage; fi

#
# Loop prep
#
c=0
SITEMAP="${SRC_DIR}/sitemap/index.txt"
TMP_WORK=`mktemp -d ${TMP_DIR}/sitemap.XXXXXX`
if [ $? -ne 0 ]; then echo "mktemp err: '${TMP_WORK}'"; exit 1; fi

mkdir -p ${SRC_DIR}/sitemap
[ -d ${SRC_DIR}/sitemap ] || \
  { echo "Can't make ${SRC_DIR}/sitemap/"; exit 1; }

if [ -f ${SITEMAP}.999 ]
  then
    rm -f ${SITEMAP}.999
  fi
for i in `seq 998 0`
  do
    if [ -f ${SITEMAP}.${i} ]
      then
        mv ${SITEMAP}.${i} ${SITEMAP}.$((i+1))
      fi
    #
  done
if [ -f ${SITEMAP} ]
  then
    mv ${SITEMAP} ${SITEMAP}.0
  fi

echo "%t ${TITLE}" > ${SITEMAP}
echo "%a ArielMT's Static Website Generator (AMTSWG)" >> ${SITEMAP}
echo "%d `date '+%A, %B %e, %Y'`" >> ${SITEMAP}
echo "" >> ${SITEMAP}
echo "m4_changequote([[\`]],[[']])m4_dnl" >> ${SITEMAP}
echo "m4_include(\`sitemap.m4')m4_dnl" >> ${SITEMAP}
echo "PRE" >> ${SITEMAP}
echo "OPEN" >> ${SITEMAP}

[ -f ${SITEMAP} ] || { "Can't save to ${SITEMAP}"; exit 1; }

echo "    Generating sitemap:"

#
# Iterate over every source file except the sitemap and backups
#
for file in `find ${SRC_DIR}/ \
                  -type f \
                  -name "*.txt" \
                  \! \
                  \( -regex "${SITEMAP}" \) \
                  -print | sort`
  do
    c=$((c+1))

    #
    # Discover file names, strip source-dir name.
    #
    fname=`echo ${file} | sed -n "s:${SRC_DIR}/::;p"`
    fbase=`echo ${fname} | sed -n "s/.txt$//;p"`
    tname="${fbase}.html"

    #
    # Discover title
    #
    sed -n "1,/^$/p" ${file} > ${TMP_WORK}/header
    grep -e "^%t " ${TMP_WORK}/header | \
    sed -n "s/^%t //;p" > ${TMP_WORK}/title
    TITLE=`cat ${TMP_WORK}/title | tr -d "\n"`
    if [ -z "${TITLE}" ]
      then
        TITLE="${tname}"
        echo -e "${c}\t${tname}"
      else
        echo -e "${c}\t${tname}\t ${TITLE}"
      fi

    #
    # Add linked listing to sitemap
    #
    echo "ITEM(\`${c}',LINK(\`${c}',\`/${tname}',\`${TITLE}'))" >> ${SITEMAP}
  done

#
# Close out sitemap
#
echo "CLOSE" >> ${SITEMAP}
echo ""      >> ${SITEMAP}

#
# Clean up
#
rm -f ${TMP_WORK}/*
rmdir ${TMP_WORK}
[ -d ${TMP_WORK} ] && \
  { echo "WARNING: ${TMP_WORK} couldn't be removed."; }
echo "done."

