#!/bin/sh

#
# Print script usage and exit
#
usage(){
  local flag=$1
  local self=`basename $0`
  cat << HERE
Synchronize destination directory's contents with generated source and
ready public contents.

Usage: ${self} <options>

-h or -?      - Show this help text and exit without error.
-s <src_dir>  - Directory with unconverted source files.
-d <dest_dir> - Directory with ready to upload generated files.
-p <pub_dir>  - Directory with public no-compile files.
-t <temp_dir> - Directory to use for temporary storage.

Directory options are either absolute paths or paths relative to the
current directory, not the directory in which ${self} lives.

HERE
  if [ -n "${flag}" ]; then exit 0; else exit 1; fi
}

#
# Basic tools check
#
which -s which || { echo "No 'which' command."; exit 1; }
which -s basename || { echo "No 'basename' command."; exit 1; }
which -s dirname || { echo "No 'dirname' command."; exit 1; }
which -s mktemp || { echo "No 'mktemp' command."; exit 1; }
[ 0 -eq 0 ]    || { echo "The '[' command doesn't work."; exit 1; } # ]
which -s grep  || { echo "No 'grep' command."; exit 1; }
which -s sed   || { echo "No 'sed' command."; exit 1; }
which -s cat   || { echo "No 'cat' command."; exit 1; }
which -s tr    || { echo "No 'tr' command."; exit 1; }
which -s find  || { echo "No 'find' command."; exit 1; }
which -s sort  || { echo "No 'sort' command."; exit 1; }
which -s comm  || { echo "No 'comm' command."; exit 1; }

#
# Process command line options
#
unset SRC_DIR DST_DIR PUB_DIR TMP_DIR

while getopts 's:d:p:t:h?' c
  do
    case $c in
      s) SRC_DIR=${OPTARG} ;;
      d) DST_DIR=${OPTARG} ;;
      p) PUB_DIR=${OPTARG} ;;
      t) TMP_DIR=${OPTARG} ;;
      h)                   ;&
      ?) usage help        ;;
    esac
  done
if [ -z "${SRC_DIR}" ]; then echo "No source dir provided."; usage; fi
if [ -z "${DST_DIR}" ]; then echo "No destination dir provided."; usage; fi
if [ -z "${PUB_DIR}" ]; then echo "No public dir provided."; usage; fi
if [ -z "${TMP_DIR}" ]; then echo "No temporary dir provided."; usage; fi

#
# Loop prep
#
TMP_WORK=`mktemp -d ${TMP_DIR}/syncpub.XXXXXX`
if [ $? -ne 0 ]; then echo "mktemp err: '${TMP_WORK}'"; exit 1; fi
c=0
cat /dev/null > ${TMP_WORK}/unsorted-source
cat /dev/null > ${TMP_WORK}/unsorted-dest
cat /dev/null > ${TMP_WORK}/sorted-source
cat /dev/null > ${TMP_WORK}/sorted-dest
cat /dev/null > ${TMP_WORK}/only-source
cat /dev/null > ${TMP_WORK}/only-dest
cat /dev/null > ${TMP_WORK}/common
[ -f ${TMP_WORK}/common ] || \
  { echo "Couldn't make temp files in ${TMP_WORK}"; exit 1; }

echo "    Synchronizing ${DST_DIR}/ to match ${PUB_DIR}/ and pages"
echo "    generated from files in ${SRC_DIR}/:"

#
# Discover files in SRC_DIR whose generated content should be in DST_DIR
#
for file in `find ${SRC_DIR}/ -type f -name "*.txt" -print`
  do
    c=$((c+1))
    #
    # Discover and convert file names
    #
    fname=`echo ${file} | sed -n "s:${SRC_DIR}/::;p"`
    fbase=`echo ${fname} | sed -n "s/.txt//;p"`
    tname="${DST_DIR}/${fbase}.html"

    #
    # Add converted line to unsorted-source list
    #
    echo ${tname} >> ${TMP_WORK}/unsorted-source
    # echo -e "${c}\tLooking for ${tname}"
  done

#
# Discover files in PUB_DIR that should be in DST_DIR
#
for file in `find ${PUB_DIR}/ -type f -print`
  do
    c=$((c+1))
    #
    # Discover and convert file names
    #
    fname=`echo ${file} | sed -n "s:${PUB_DIR}/::;p"`
    tname="${DST_DIR}/${fname}"

    #
    # Add converted line to unsorted-source list
    #
    echo ${tname} >> ${TMP_WORK}/unsorted-source
    # echo -e "${c}\tLooking for ${tname}"
  done

#
# Discover files actually in DST_DIR whether they should be there or not
#
for file in `find ${DST_DIR}/ -type f -print`
  do
    c=$((c+1))
    echo ${file} >> ${TMP_WORK}/unsorted-dest
    # echo -e "${c}\tFound ${file}"
  done

#
# Sort and separate lists
#
ssource="${TMP_WORK}/sorted-source"
sdest="${TMP_WORK}/sorted-dest"
sort ${TMP_WORK}/unsorted-source > ${ssource}
sort ${TMP_WORK}/unsorted-dest   > ${sdest}
comm -12 ${ssource} ${sdest} > ${TMP_WORK}/common
comm -13 ${ssource} ${sdest} > ${TMP_WORK}/only-dest
comm -23 ${ssource} ${sdest} > ${TMP_WORK}/only-source

#
# Delete files in DST_DIR that shouldn't be there
#
cat ${TMP_WORK}/only-dest | while read file
  do
    c=$((c+1))
    # printf "${c}\tDeleting ${file} ... "
    printf "Deleting orphaned ${file} ... "
    rm -f ${file}
    rmdir -p `dirname ${file}` 2> /dev/null
    [ -f ${file} ] && { echo "can't delete."; exit 1; }
    echo "done."
  done

#
# For every file that should be in DST_DIR but isn't, if it's in
# PUB_DIR, then copy it to DST_DIR
#
cat ${TMP_WORK}/only-source | while read file
  do
    c=$((c+1))
    #
    # Discover and convert file name
    #
    tname=`echo ${file} | sed -n "s:${DST_DIR}/::;p"`
    fname="${PUB_DIR}/${tname}"

    #
    # Find and copy the public file
    #
    if [ -f "${fname}" ]
      then
        # printf "${c}\tCopying missing ${file} ... "
        printf "Copying missing ${file} ... "
        mkdir -p `dirname ${file}`
        cp -af ${fname} ${file}
        [ -f ${file} ] || { echo "can't copy."; exit 1; }
        echo "done."
      else
        # echo -e "${c}\tDid not copy missing ${file}"
        echo -e "Did not copy missing ${file}"
        # (because it's generated from SRC_DIR, not in PUB_DIR)
      fi
    #
  done

#
# For every file in DST_DIR that should be there, make sure it's up to
# date
#
cat ${TMP_WORK}/common | while read file
  do
    c=$((c+1))
    #
    # Discover and convert file name
    #
    tname=`echo ${file} | sed -n "s:${DST_DIR}/::;p"`
    fname="${PUB_DIR}/${tname}"

    #
    # Check if PUB_DIR version is newer than DST_DIR version
    #
    if [ "${fname}" -nt "${file}" ]
      then
        # printf "${c}\tCopying newer ${file} ... "
        printf "Copying newer ${file} ... "
        cp -af ${fname} ${file}
        [ "${fname}" -nt "${file}" ] && { echo "can't copy."; exit 1; }
        echo "done."
      else
        # echo -e "${c}\tLeft ${file}"
        # (because PUB_DIR is not newer or it's generated from SRC_DIR.)
      fi
    #
  done

#
# Clean up
#
rm -f ${TMP_WORK}/*
rmdir ${TMP_WORK}
[ -d ${TMP_WORK} ] && \
  { echo "WARNING: ${TMP_WORK} couldn't be removed."; }
echo "done."

