#!/bin/sh

#
# Print script usage and exit
#
usage(){
  local flag=$1
  local self=`basename $0`
  cat << HERE
Convert a source file from LML or HTML fragment to a full HTML page.

Usage: ${self} <options>

Options:

-h or -?      - Show this help text and exit without error.
-s <src_dir>  - Directory containing source file to convert.
-d <dest_dir> - Directory containing the converted destination file.
-t <temp_dir> - Directory to use for temporary storage.
-i <tpl_dir>  - Directory containing site templates.
-x <command>  - Lightweight markup translation command name.
-1 <options>  - Translation options between command and input file name.
-2 <options>  - Translation options between input and output file names.
-3 <options>  - Translation options after output file name.
-f <filename> - Name of source file to convert, without source dir name.

When the lightweight markup to HTML fragment command is invoked, the -x,
-1, -2, and -3 options are assembled and run as follows:

[-x command] [-1 options] [input file] [-2 options] [output file] [-3 options]

If no translation needs to be done (that is, the input file is already
an HTML fragment), then the -x option should be "-x cp", the -1 option
should be "-1 '-f'", and the -2 and -3 options should not be provided.

Directory options are either absolute paths or paths relative to the
current directory, not the directory in which ${self} lives.

HERE
  if [ -n "${flag}" ]; then exit 0; else exit 1; fi
}

#
# Basic tools check
#
which -s which || { echo "No 'which' command."; exit 1; }
which -s basename || { echo "No 'basename' command."; exit 1; }
which -s dirname || { echo "No 'dirname' command."; exit 1; }
which -s mktemp || { echo "No 'mktemp' command."; exit 1; }
[ 0 -eq 0 ]    || { echo "The '[' command doesn't work."; exit 1; } # ]
which -s grep  || { echo "No 'grep' command."; exit 1; }
which -s sed   || { echo "No 'sed' command."; exit 1; }
which -s cat   || { echo "No 'cat' command."; exit 1; }
which -s tr    || { echo "No 'tr' command."; exit 1; }
which -s find  || { echo "No 'find' command."; exit 1; }
which -s sort  || { echo "No 'sort' command."; exit 1; }
which -s comm  || { echo "No 'comm' command."; exit 1; }
which -s m4    || { echo "No 'm4' command."; exit 1; }

#
# Process command line options
#
unset SRC_DIR DST_DIR TMP_DIR TPL_DIR FILE
unset XLAT XLAT_OPTS1 XLAT_OPTS2 XLAT_OPTS3

while getopts 's:d:t:i:x:1:2:3:f:h?' c
  do
    case $c in
      s) SRC_DIR=${OPTARG}    ;;
      d) DST_DIR=${OPTARG}    ;;
      t) TMP_DIR=${OPTARG}    ;;
      i) TPL_DIR=${OPTARG}    ;;
      x) XLAT=${OPTARG}       ;;
      1) XLAT_OPTS1=${OPTARG} ;;
      2) XLAT_OPTS2=${OPTARG} ;;
      3) XLAT_OPTS3=${OPTARG} ;;
      f) FILE=${OPTARG}       ;;
      h)                      ;&
      ?) usage help           ;;
    esac
  done
if [ -z "${FILE}" ]; then echo "No filename provided."; usage; fi
if [ -z "${TMP_DIR}" ]; then echo "No temp dir provided."; usage; fi
if [ -z "${DST_DIR}" ]; then echo "No destination dir provided."; usage; fi
if [ -z "${SRC_DIR}" ]; then echo "No source dir provided."; usage; fi
if [ -z "${TPL_DIR}" ]; then echo "No templates dir provided."; usage; fi
if [ -z "${XLAT}" ]; then echo "No translation command provided."; usage; fi

OFILE="`echo ${FILE} | sed -n "s/.txt$//;p"`.html"
mkdir -p `dirname ${DST_DIR}/${OFILE}`
echo ""
echo -n "==> ${SRC_DIR}/${FILE} --> ${DST_DIR}/${OFILE}: "

#
# Extract pieces from the source file
#
TMP_WORK=`mktemp -d ${TMP_DIR}/txt2html.XXXXXX`
if [ $? -ne 0 ]; then echo "mktemp err: '${TMP_WORK}'"; exit 1; fi

sed -n "1,/^$/p" ${SRC_DIR}/${FILE} > ${TMP_WORK}/header

#
# For M4 preprocessing: change quotes so we can use apostrophes in the
# body text.
#
echo "m4_changequote(\`[[',\`]]')m4_dnl" > ${TMP_WORK}/body
echo "m4_changecom()m4_dnl" >> ${TMP_WORK}/body
sed -n "1,/^$/d;p" ${SRC_DIR}/${FILE} >> ${TMP_WORK}/body

#
# %t Page Title
# used in the HTML head as well as for the page content title.
#
grep -e "^%t " ${TMP_WORK}/header | \
sed -n "s/^%t //;p" | tr -d "\n" > ${TMP_WORK}/title

#
# %a Jane Doe
# credits for page content.
#
grep -e "^%a " ${TMP_WORK}/header | \
sed -n "s/^%a //;p" | tr -d "\n" > ${TMP_WORK}/author

#
# %d Monday, January 5, 2099
# Every page should have the date it was written.  It eliminates any
# doubt about information timeliness or context.
#
grep -e "^%d " ${TMP_WORK}/header | \
sed -n "s/^%d //;p" | tr -d "\n" > ${TMP_WORK}/date

#
# %tpl template-file-name.m4
# M4 template in the templates directory to use instead of "page.m4"
#
grep -e "^%tpl " ${TMP_WORK}/header | \
sed -n "s/^%tpl //;p" | tr -d "\n" > ${TMP_WORK}/template

#
# Set flags for processing based on file sizes
#
TEST=`cat ${TMP_WORK}/title | tr -d "\n"`
if [ -n "${TEST}" ]; then USE_TITLE=1; else USE_TITLE=0; fi
TEST=`cat ${TMP_WORK}/author | tr -d "\n"`
if [ -n "${TEST}" ]; then USE_AUTHOR=1; else USE_AUTHOR=0; fi
TEST=`cat ${TMP_WORK}/date | tr -d "\n"`
if [ -n "${TEST}" ]; then USE_DATE=1; else USE_DATE=0; fi
TEMPLATE=`cat ${TMP_WORK}/template | tr -d "\n"`
if [ -z "${TEMPLATE}" ]; then TEMPLATE="page.m4"; fi

#
# Preprocess the body text with m4
#
m4="-P"
if [ $USE_TITLE = 1 ]; then m4="${m4} -DTITLE=${TMP_WORK}/title"; fi
if [ $USE_AUTHOR = 1 ]; then m4="${m4} -DAUTHOR=${TMP_WORK}/author"; fi
if [ $USE_DATE = 1 ]; then m4="${m4} -DDATE=${TMP_WORK}/date"; fi
m4a="${m4} -DTEMPLATE=${TPL_DIR}/${TEMPLATE} -I${TPL_DIR}/ -I${SRC_DIR}/"
dn="${FILE}"
while [ "${dn}" != "." ]
  do
    dn=`dirname ${dn}`
    if [ "${dn}" != "." ]
      then
        m4a="${m4a} -I${SRC_DIR}/${dn}/"
      fi
    #
  done
#m4a="${m4a} -d V";echo "m4...";sleep 3
#echo "m4 ${m4a} ${TMP_WORK}/body > ${TMP_WORK}/source"
echo -n "m4 preprocessing,"
m4 ${m4a} ${TMP_WORK}/body > ${TMP_WORK}/source
if [ $? -ne 0 ]
  then
    echo "    M4 preprocessing error."
    echo "If the error is quotation-related, consider adding another"
    echo "    layer of quotes in the macro causing the error."
    exit 1
  fi

#
# Translate the preprocessed source text to an HTML fragment.
#
echo -n " ${XLAT},"
${XLAT} ${XLAT_OPTS1} \
        ${TMP_WORK}/source \
        ${XLAT_OPTS2} \
        ${TMP_WORK}/html \
        ${XLAT_OPTS3}

#
# Run HTML fragment through the M4 template to produce the final page.
#
m4a="${m4}"
if [ -n "`cat ${TMP_WORK}/html | tr -d "\n"`" ]
  then
    m4a="${m4a} -DBODY=${TMP_WORK}/html"
  fi
m4a="${m4a} -I${TPL_DIR}/ -I${SRC_DIR}/"
dn="${FILE}"
while [ "${dn}" != "." ]
  do
    dn=`dirname ${dn}`
    if [ "${dn}" != "." ]
      then
        m4a="${m4a} -I${SRC_DIR}/${dn}/"
      fi
    #
  done
#echo "m4 ${m4a} ${TPL_DIR}/${TEMPLATE} > ${DST_DIR}/${OFILE}"
echo -n " m4 postprocessing,"
m4 ${m4a} ${TPL_DIR}/${TEMPLATE} > ${DST_DIR}/${OFILE}
if [ $? -ne 0 ]
  then
    echo "M4 postprocessing error."
    echo "Did you typo the template filename?  Does the template file"
    echo "exist where you think it does?"
    exit 1
  fi

#
# Clean up.
#
rm -f ${TMP_WORK}/*
rmdir ${TMP_WORK}
[ -d ${TMP_WORK} ] && \
  { echo -n " WARNING: ${TMP_WORK} couldn't be removed,"; }
echo " done."
echo ""

